@@ -55,7 +55,7 @@ require ("string.js");
 // ';' is for align `for' parts
 // ' ' is to add a '()' after `if', `while', `for', ...
 // TBD <others>
-triggerCharacters = "{}()<>/:;,#\\?|/%.@ ";
+triggerCharacters = "{}()<>/:;,#\\?!|/%.@ \"";
 
 var debugMode = false;
 
@@ -740,6 +740,50 @@ function tryPreprocessor_ch(line)
     return result;
 }
 
+/**
+ * Check if \c ENTER was pressed on a start of line and
+ * after a block comment.
+ */
+function tryAfterBlockComment_ch(line)
+{
+    var result = -1;
+    if (0 < line)
+    {
+        var prev_non_empty_line = document.prevNonEmptyLine(line - 1);
+        if (prev_non_empty_line != -1 && document.line(prev_non_empty_line).trim().startsWith("*/"))
+        {
+            var p = document.firstColumn(prev_non_empty_line);
+            if ((p % gIndentWidth) != 0)
+                result = Math.floor(p / gIndentWidth) * gIndentWidth;
+        }
+    }
+    if (result != -1)
+    {
+        dbg("tryAfterBlockComment_ch result="+result);
+    }
+    return result;
+}
+
+/**
+ * Check if \c ENTER was pressed after \c break or \c continue statements
+ * and if so, unindent the current line.
+ */
+function tryAfterBreakContinue_ch(line)
+{
+    var result = -1;
+    var currentLineText = document.line(line - 1).ltrim();
+    var should_proceed = currentLineText.startsWith("break;") || currentLineText.startsWith("continue;")
+    if (should_proceed)
+    {
+        result = document.firstColumn(line - 1) - gIndentWidth;
+    }
+    if (result != -1)
+    {
+        dbg("tryAfterBreakContinue_ch result="+result);
+    }
+    return result;
+}
+
 /// Wrap \c tryToKeepInlineComment as \e caret-handler
 function tryToKeepInlineComment_ch(line)
 {
@@ -776,6 +820,8 @@ function caretPressed(cursor)
       , tryBeforeAccessSpecifier_ch
       , tryAfterEqualChar_ch
       , tryPreprocessor_ch
+      , tryAfterBlockComment_ch
+      , tryAfterBreakContinue_ch
       , tryToKeepInlineComment_ch                           // NOTE This must be a last checker!
     ];
 
@@ -947,6 +993,13 @@ function tryComma(cursor)
     return result;
 }
 
+/**
+ * \brief Move towards a document start and look for control flow keywords
+ *
+ * \note Keyword must be at the line start
+ *
+ * \return found line's indent, otherwise \c -2 if nothing found
+ */
 function tryBreakContinue(line, is_break)
 {
     var result = -2;
@@ -974,12 +1027,37 @@ function tryBreakContinue(line, is_break)
     return result;
 }
 
+/**
+ * \brief Handle \c ; character.
+ *
+ * Here is few cases possible (handled):
+ * \li semicolon is a first char on a line -- then, it looks like \c for statement
+ *      splitted accross the lines
+ * \li semicolon entered after some keywords: \c break or \c continue, then we
+ *      need to align this line taking in account a previous one
+ * \li and finally here is a trick: when auto brackets extension enabled, and user types
+ *      a function call like this:
+ * \code
+ *  auto var = some_call(arg1, arg2|)
+ * \endcode
+ * (\c '|' shows a cursor position). Note there is no final semicolon in this expression,
+ * cuz pressing <tt>'('</tt> leads to the following snippet: <tt>some_call(|)</tt>, so to
+ * add a semicolon you have to move cursor out of parenthesis. The trick is to allow to press
+ * <tt>';'</tt> at position shown in the code snippet, so indenter will transform it into this:
+ * \code
+ *  auto var = some_call(arg1, arg2);|
+ * \endcode
+ */
 function trySemicolon(cursor)
 {
     var result = -2;
     var line = cursor.line;
     var column = cursor.column;
 
+    if (isStringOrComment(line, column))
+        return result;                                      // Do nothing for comments and strings
+
+    // If ';' is a first char on a line?
     if (document.firstChar(line) == ';' && document.firstColumn(line) == (column - 1))
     {
         // Check if we are inside a `for' statement
@@ -1002,6 +1080,30 @@ function trySemicolon(cursor)
             if (result == -2)
                 result = -1;
         }
+        else
+        {
+            // Check if next character(s) is ')' and nothing after
+            should_proceed = true;
+            var lineLength = document.lineLength(line);
+            for (var i = column; i < lineLength; ++i)
+            {
+                if (document.charAt(line, i) != ')')
+                {
+                    should_proceed = false;
+                    break;
+                }
+            }
+            if (should_proceed)
+            {
+                document.editBegin();
+                // Remove ';' from column - 1
+                document.removeText(line, column - 1, line, column);
+                // Append ';' to the end of line
+                document.insertText(line, lineLength - 1, ";");
+                document.editEnd();
+                view.setCursorPosition(line, lineLength);
+            }
+        }
     }
     return result;
 }
@@ -1108,11 +1210,12 @@ function tryCloseBracket(cursor, ch)
 }
 
 /**
- * \brief Indent new scope block
+ * \brief Indent a new scope block
  *
  * ... try to unindent to be precise... First of all check that open
- * \c '{' is a first symbol on a line, and if it doesn't, make sure
- * there is a space before, except if previous char is not a \c '{'.
+ * \c '{' is a first symbol on a line, and if it doesn't,
+ * add space (if absent at previous position) after <tt>')'</tt> or \c '='
+ * or if line stats w/ some keywords: \c enum, \c class, \c struct or \c union.
  * Otherwise, look at the previous line for dangling <tt>')'</tt> or
  * a line started w/ one of flow control keywords.
  *
@@ -1146,13 +1249,24 @@ function tryBlock(cursor)
     }
     else
     {
-        // '{' is not a first char. Check for previous one...
+        // '{' is not a first char. Check for a previous one...
         if (1 < column)
         {
             var prevChar = document.charAt(line, column - 2);
             dbg("tryBlock: prevChar='"+prevChar+"'");
-            if (prevChar != ' ' && prevChar != '{' && prevChar != '(')
+            if (prevChar == ')' || prevChar == '=')
                 document.insertText(line, column - 1, ' ');
+            else if (prevChar != ' ')
+            {
+                var currentLine = document.line(line).ltrim();
+                var starts_with_keyword = currentLine.startsWith('struct ')
+                  || currentLine.startsWith('class ')
+                  || currentLine.startsWith('union ')
+                  || currentLine.startsWith('enum ')
+                  ;
+                if (starts_with_keyword)
+                    document.insertText(line, column - 1, ' ');
+            }
         }
     }
     return result;
@@ -1368,6 +1482,94 @@ function tryDoxygenGrouping(cursor)
     }
 }
 
+/**
+ * \brief Handle quote character
+ *
+ * Look back for \c 'R' char right before \c '"' and if
+ * the next one (after \c '"') is not an alphanumeric,
+ * then add a delimiters.
+ *
+ * \attention If autobrace plugin used to add quotation marks,
+ * then next symbol after the current will be another one \c '"'
+ * if this is a new string. Otherwise, if it turned OFF, the simplest
+ * case we can detect (and the most typical), is that raw string started
+ * at fresh line, and current \c column equal to the line length.
+ */
+function tryRawStringLiteral(cursor)
+{
+    var line = cursor.line;
+    var column = cursor.column;
+    var is_abp_on = document.charAt(line, column) == '"';
+    var start_raw_string = 2 <= column
+      && document.charAt(line, column - 2) == 'R'
+      && (
+          is_abp_on                                         // Autobracket plugin On?
+        || column == (document.lineLength(line))            // Autobracket plugin Off? (and end of line)
+        );
+    if (start_raw_string)
+    {
+        document.editBegin();
+        /// \todo Make delimiter configurable... HOW?
+        /// It would be nice if indenters can have a configuration page somehow...
+        document.insertText(cursor, "~()~");
+        if (!is_abp_on)
+            document.insertText(line, column + 4, '"');
+        view.setCursorPosition(line, column + 2);
+        document.editEnd();
+    }
+}
+
+/**
+ * \brief Handle \c '!' char
+ *
+ * Exclamation symbol can be a part of \c operator!= or unary operator.
+ * in both cases, a space required before it! Except few cases:
+ * - when it is at the line start
+ * - when a char before it \c '(' -- i.e. argument of a control flow keyword (\c if, \c while)
+ *   or a function call parameter
+ * - when a char before it \c '[' (array subscript)
+ * - when a char before it \c '<' -- here is two cases possible:
+ *      - it is a first non-type template parameter (w/ type \c bool obviously)
+ *      - it is a part of less or shift operators.
+ * To distinct last case, it is enough to check that a word before \c '<' (w/o space)
+ * is an identifier.
+ * \note Yep, operators supposed to be separated from around text.
+ */
+function tryExclamation(cursor)
+{
+    var line = cursor.line;
+    var column = cursor.column;
+
+    // Do nothing for comments and stings
+    if (isStringOrComment(line, column - 1))
+        return;
+
+    // Make sure '!' is not a first char on a line
+    if (document.firstColumn(line) == column - 1)
+        return;
+
+    // Do nothing if one of 'stop' chars:
+    var prev_char = document.charAt(line, column - 2);
+    dbg("prev_char: ",prev_char);
+    if (prev_char == ' ' || prev_char == '(' || prev_char == '[')
+        return;
+
+    // And finally make sure it is not a part of 'relation operator'
+    if (prev_char == '<')
+    {
+        // Make sure a char before is not a space or another '<'
+        var prev_prev_char = document.charAt(line, column - 3);
+        dbg("prev_prev_char: ",prev_prev_char);
+        if (prev_prev_char != ' ' && prev_prev_char != '<')
+            return;
+    }
+
+    // Ok, if we r here, just insert a space ;)
+    document.editBegin();
+    document.insertText(line, column - 1, " ");
+    document.editEnd();
+}
+
 /**
  * \brief Handle a space
  *
@@ -1384,6 +1586,7 @@ function tryKeywordsWithBrackets(cursor)
       || text == "while "
       || text == "for "
       || text == "switch "
+      || text == "catch "
       ;
     if (need_brackets)
     {
@@ -1462,6 +1665,12 @@ function processChar(line, ch)
         case '@':
             tryDoxygenGrouping(cursor);
             break;
+        case '"':
+            tryRawStringLiteral(cursor);
+            break;
+        case '!':                                           // Almost all the time there should be a space before!
+            tryExclamation(cursor);
+            break;
         case ' ':
             tryKeywordsWithBrackets(cursor);
             break;
