@@ -40,6 +40,7 @@
 
 #include "kateviinputmodemanager.h"
 #include "katetextrange.h"
+#include "katetextfolding.h"
 #include "katerenderer.h"
 
 namespace KTextEditor
@@ -57,7 +58,7 @@ class KateSpellCheckDialog;
 class KateCompletionWidget;
 class KateViewInternal;
 class KateSearchBar;
-class KateViModeBar;
+class KateViEmulatedCommandBar;
 class KateViewBar;
 class KateGotoBar;
 class KateDictionaryBar;
@@ -67,7 +68,6 @@ class KateMessageWidget;
 
 class KToggleAction;
 class KAction;
-class KRecentFilesAction;
 class KSelectAction;
 
 class QVBoxLayout;
@@ -190,6 +190,20 @@ class KATEPART_TESTS_EXPORT KateView : public KTextEditor::View,
 
   Q_SIGNALS:
     void configChanged();
+    
+  public:
+    /**
+     * Try to fold starting at the given line.
+     * This will both try to fold existing folding ranges of this line and to query the highlighting what to fold.
+     * @param startLine start line to fold at
+     */
+    void foldLine (int startLine);
+
+     /**
+     * Try to unfold all foldings starting at the given line.
+     * @param startLine start line to unfold at
+     */
+    void unfoldLine (int startLine);
 
   //
   // KTextEditor::CodeCompletionInterface2
@@ -380,7 +394,7 @@ class KATEPART_TESTS_EXPORT KateView : public KTextEditor::View,
      Return values for "save" related commands.
     */
     bool isOverwriteMode() const;
-    enum KTextEditor::View::EditMode viewEditMode() const {return isOverwriteMode() ? KTextEditor::View::EditOverwrite : KTextEditor::View::EditInsert;}
+    EditMode viewEditMode() const;
     QString currentTextLine();
 
     /**
@@ -514,8 +528,9 @@ class KATEPART_TESTS_EXPORT KateView : public KTextEditor::View,
     void toggleDynWordWrap ();
     void toggleViInputMode ();
 
-    void showViModeBar ();
-    void hideViModeBar ();
+
+    void showViModeEmulatedCommandBar();
+
     void setDynWrapIndicators(int mode);
 
   public:
@@ -560,6 +575,16 @@ class KATEPART_TESTS_EXPORT KateView : public KTextEditor::View,
   Q_SIGNALS:
     void dropEventPass(QDropEvent*);
 
+  public:
+    /**
+     * Folding handler for this view.
+     * @return folding handler
+     */
+    Kate::TextFolding &textFolding ()
+    {
+      return m_textFolding;
+    }
+
   public:
     void slotTextInserted ( KTextEditor::View *view, const KTextEditor::Cursor &position, const QString &text);
 
@@ -571,11 +596,14 @@ class KATEPART_TESTS_EXPORT KateView : public KTextEditor::View,
     void slotLostFocus();
     void slotDropEventPass( QDropEvent* ev );
     void slotSaveCanceled( const QString& error );
+    void slotConfigDialog ();
+
+  public Q_SLOTS: // TODO: turn into good interface, see kte5/foldinginterface.h
+    void slotFoldToplevelNodes();
     void slotCollapseLocal();
     void slotCollapseLevel();
     void slotExpandLevel();
     void slotExpandLocal();
-    void slotConfigDialog ();
 
   private:
     void setupConnections();
@@ -587,7 +615,6 @@ class KATEPART_TESTS_EXPORT KateView : public KTextEditor::View,
     KAction*               m_editUndo;
     KAction*               m_editRedo;
     KAction*               m_pasteMenu;
-    KRecentFilesAction*    m_fileRecent;
     KToggleAction*         m_toggleFoldingMarkers;
     KToggleAction*         m_toggleIconBar;
     KToggleAction*         m_toggleLineNumbers;
@@ -617,6 +644,7 @@ class KATEPART_TESTS_EXPORT KateView : public KTextEditor::View,
     bool m_hasWrap;
 
     KateDocument     *const m_doc;
+    Kate::TextFolding m_textFolding;
     KateViewConfig   *const m_config;
     KateRenderer     *const m_renderer;
     KateViewInternal *const m_viewInternal;
@@ -668,11 +696,11 @@ class KATEPART_TESTS_EXPORT KateView : public KTextEditor::View,
     KateCommandLineBar *cmdLineBar ();
     KateScriptConsole *consoleBar ();
     KateDictionaryBar *dictionaryBar();
+    KateViEmulatedCommandBar *viModeEmulatedCommandBar();
 
   private:
     KateSearchBar *searchBar (bool initHintAsPower = false);
     bool hasSearchBar () const { return m_searchBar != 0; }
-    KateViModeBar *viModeBar();
     KateGotoBar *gotoBar ();
 
   /**
@@ -687,7 +715,7 @@ class KATEPART_TESTS_EXPORT KateView : public KTextEditor::View,
     KateCommandLineBar *m_cmdLine;
     KateScriptConsole *m_console;
     KateSearchBar *m_searchBar;
-    KateViModeBar *m_viModeBar;
+    KateViEmulatedCommandBar *m_viModeEmulatedCommandBar;
     KateGotoBar *m_gotoBar;
     KateDictionaryBar *m_dictionaryBar;
 
@@ -750,6 +778,23 @@ class KATEPART_TESTS_EXPORT KateView : public KTextEditor::View,
     KateSpellingMenu* spellingMenu();
   private:
     bool m_userContextMenuSet;
+    
+  private Q_SLOTS:
+    /**
+     * save folding state before document reload
+     */
+    void saveFoldingState ();
+    
+    /**
+     * restore folding state after document reload
+     */
+    void applyFoldingState ();
+    
+  private:
+    /**
+     * saved folding state
+     */
+    QVariantList m_savedFoldingState;
 
 public:
     /**
@@ -830,18 +875,20 @@ public:
     void postMessage(KTextEditor::Message* message, QList<QSharedPointer<QAction> > actions);
 
   private:
-    /**
-     * Message widget showing KTextEditor::Messages above the View.
-     */
+    /** Message widget showing KTextEditor::Messages above the View. */
     KateMessageWidget* m_topMessageWidget;
-    /**
-     * Message widget showing KTextEditor::Messages below the View.
-     */
+    /** Message widget showing KTextEditor::Messages below the View. */
     KateMessageWidget* m_bottomMessageWidget;
-    /**
-     * Message widget showing KTextEditor::Messages as view overlay.
-     */
-    KateMessageWidget* m_overlayMessageWidget;
+    /** Message widget showing KTextEditor::Messages as view overlay in top right corner. */
+    KateMessageWidget* m_floatTopMessageWidget;
+    /** Message widget showing KTextEditor::Messages as view overlay in bottom left corner. */
+    KateMessageWidget* m_floatBottomMessageWidget;
+    /** Layout for floating notifications */
+    QVBoxLayout* m_notificationLayout;
+
+  // for unit test 'tests/messagetest.cpp'
+  public:
+    KateMessageWidget* messageWidget() { return m_floatTopMessageWidget; }
 };
 
 /**
